# frozen_string_literal: true

require "nokogiri"
require "dhall"

require "dhall/xml/as_dhall"
require "dhall/xml/coder"

module Dhall
	module XML
		TYPE = Forall.of_arguments(
			Builtins[:Type],
			RecordType.new(
				record: {
					"element" => Forall.of_arguments(
						RecordType.new(
							record: {
								"attributes" => Builtins[:List].(RecordType.new(
									record: {
										"mapKey"   => Builtins[:Text],
										"mapValue" => Builtins[:Text]
									}
								)),
								"content"    => Builtins[:List].(Variable["_"]),
								"name"       => Builtins[:Text]
							}
						),
						body: Variable["_", 1]
					),
					"text"    => Forall.of_arguments(Builtins[:Text], body: Variable["_", 1])
				}
			),
			body: Variable["_", 1]
		).freeze

		def self.from_dhall(expr, doc=Nokogiri::XML::Document.new)
			return expr.unwrap if expr.is_a?(RubyObjectRaw)

			TypeChecker.annotate(
				TypeAnnotation.new(value: expr, type: TYPE)
			).value.(
				TYPE, # dummy
				element: FunctionProxyRaw.new(->(record) { from_element(record, doc) }),
				text:    FunctionProxyRaw.new(doc.method(:create_text_node))
			).unwrap
		end

		def self.from_element(record, doc=Nokogiri::XML::Document.new)
			doc.create_element(
				record.fetch(:name).to_s,
				Coder.new.decode(record.fetch(:attributes))
			) do |el|
				el.add_child(Nokogiri::XML::NodeSet.new(
					doc,
					record.fetch(:content).to_a.map { |xml| from_dhall(xml, doc) }
				))
			end
		end
	end
end
