# frozen_string_literal: true

require "nokogiri"
require "dhall"

module Dhall
	module XML
		module AsDhall
			refine Nokogiri::XML::CharacterData do
				def dhall_expression(var)
					RecordSelection.new(
						record:   var,
						selector: "text"
					).(Text.new(value: text))
				end

				def as_dhall
					Dhall::Function.of_arguments(
						TYPE.type,
						TYPE.body.type,
						body: dhall_expression(Variable["_"])
					)
				end
			end

			refine Nokogiri::XML::Node do
				using Dhall::AsDhall

				def dhall_expression(var)
					RecordSelection.new(record: var, selector: "element").({
						name:       name,
						attributes: dhallish_attributes,
						content:    dhallish_children
					}.as_dhall)
				end

				def as_dhall
					Function.of_arguments(
						TYPE.type,
						TYPE.body.type,
						body: dhall_expression(Variable["_"])
					)
				end

				def dhallish_attributes
					List.of(*attributes.to_a.map do |(attr, val)|
						{ mapKey: attr.to_s, mapValue: val.to_s }.as_dhall
					end, type: Types::MAP_ENTRY())
				end

				def dhallish_children
					List.of(*children.map do |c|
						c.dhall_expression(Variable["_"])
					end, type: Variable["_", 1])
				end
			end

			refine Nokogiri::XML::Document do
				def as_dhall
					root.as_dhall
				end
			end
		end
	end
end
