# frozen_string_literal: true

require "nokogiri"
require "dhall"

require "dhall/xml/as_dhall"

module Dhall
	module XML
		class Coder < Dhall::Coder
			using Dhall::AsDhall
			using Dhall::XML::AsDhall

			def initialize(default: nil, safe: JSON_LIKE + [Nokogiri::XML::Node])
				super
			end

			def decode(expr, *args)
				if expr.is_a?(Function) && TypeChecker.type_of(expr) == TYPE
					def expr.to_ruby
						XML.from_dhall(self)
					end
				end

				super
			end

			def dump(obj)
				return if obj.nil?

				CBOR.encode(@verifier.verify(obj, "dump").as_dhall)
			end
		end
	end
end
